@@ -23,7 +23,7 @@ use XML::Smart::Tie ;
 use XML::Smart::Tree ;
 
 our ($VERSION) ;
-$VERSION = '1.6.1' ;
+$VERSION = '1.6.2' ;
 
 ###############
 # AUTOLOADERS #
@@ -1346,7 +1346,7 @@ B<Examples of use:>
              }
   ) ;
 
-=head2  apply_dtd (DTD)
+=head2  apply_dtd (DTD , OPTIONS)
 
 Apply the I<DTD> to the XML tree.
 
@@ -1357,6 +1357,34 @@ formated in a specific DTD, so, elements will be nodes automatically,
 attributes will be checked, required elements and attributes will be created,
 the element order will be set, etc...
 
+B<OPTIONS:>
+
+=over 10
+
+=item no_delete BOOL
+
+If TRUE tells that not defined elements and attributes in the DTD won't be deleted
+from the XML tree.
+
+=back
+
+B<Example of use:>
+
+  $xml->apply_dtd(q`
+  <!DOCTYPE cds [
+  <!ELEMENT cds (album+)>
+  <!ATTLIST cds
+            creator  CDATA
+            date     CDATA #REQUIRED
+            type     (a|b|c) #REQUIRED "a"
+  >
+  <!ELEMENT album (#PCDATA)>
+  ]>
+  ` ,
+  no_delete => 1 ,
+  );
+
+
 =head2  args()
 
 Return the arguments names (not nodes).
