@@ -130,7 +130,15 @@ sub data {
   
   if ( $args{noheader} ) { $xml = '' ; $metagen = '' if $args{nometagen} eq '' ;}
   
-  $data = $xml . $metagen . $meta . $data ;
+  my $dtd ;
+  
+  if ( !$args{nodtd} && $$this->{DTD} ) {
+    $dtd = ref $$this->{DTD} ? $$this->{DTD}->CutDTD : $$this->{DTD} ;
+    $dtd =~ s/\s*$// ;
+    $dtd = "\n$dtd" if $dtd ne '' && !$args{nospace} ;
+  }
+  
+  $data = $xml . $metagen . $meta . $dtd . $data ;
   
   if ($xml eq '') { $data =~ s/^\s+//gs ;}
   
@@ -191,10 +199,12 @@ sub _is_unicode {
 sub _data {
   my ( $data , $tree , $tag , $level , $prev_tree , $parsed , $ar_i , $node_type , @stat ) = @_ ;
 
-  if (ref($tree) eq 'XML::Smart') { $tree = $$tree->{point} ;}
+  if (ref($tree) eq 'XML::Smart') { $tree = defined $$tree->{content} ? $$tree->{content} : $$tree->{point} ;}
   
-  if ($$parsed{"$tree"}) { return ;}
-  $$parsed{"$tree"}++ ;
+  if ( ref($tree) ) {
+    if ($$parsed{"$tree"}) { return ;}
+    ++$$parsed{"$tree"} ;
+  }
   
   my $ident = "\n" ;
   $ident .= '  ' x $level if !$stat[0] ;
@@ -215,7 +225,7 @@ sub _data {
     if ( !$stat[5] && $tree->{'/order'} ) {
       my %keys ;
       foreach my $keys_i ( @{$tree->{'/order'}} ) {
-        if ( exists $$tree{$keys_i} && (!ref($$tree{$keys_i}) || ref($$tree{$keys_i}) eq 'HASH' || (ref($$tree{$keys_i}) eq 'ARRAY' && exists $$tree{$keys_i}[ $keys{$keys_i} ] ) ) ) {        
+        if ( exists $$tree{$keys_i} && (!ref($$tree{$keys_i}) || ref($$tree{$keys_i}) eq 'HASH' || ref($$tree{$keys_i}) eq 'XML::Smart' || (ref($$tree{$keys_i}) eq 'ARRAY' && exists $$tree{$keys_i}[ $keys{$keys_i} ] ) ) ) {
           push(@all_keys , $keys_i) ;
           
           if ( ++$keys{$keys_i} == 2 && ref $$tree{$keys_i} eq 'ARRAY' ) {
@@ -234,11 +244,18 @@ sub _data {
 
     foreach my $Key ( @all_keys ) {
       if ($Key eq '' || $Key eq '/order' || $Key eq '/nodes') { next ;}
-      
-      if (ref($$tree{$Key})) {
+
+      if ( $Key eq '!--' && (!ref($$tree{$Key}) || ( ref($$tree{$Key}) eq 'HASH' && (keys %{$$tree{$Key}}) == 1 && (defined $$tree{$Key}{CONTENT} || defined $$tree{$Key}{content}) ) ) ) {
+        my $ct = $$tree{$Key} ;
+        if (ref $$tree{$Key}) { $ct = defined $$tree{$Key}{CONTENT} ? $$tree{$Key}{CONTENT} : $$tree{$Key}{content} ;} ;
+        if ( $ct ne '' ) { $tags .= "$ident<!-- $ct -->" ;}
+      }
+      elsif (ref($$tree{$Key})) {
         my $k = $$tree{$Key} ;
         my $i ;
-        if (ref $k eq 'XML::Smart') { $k = ${$$tree{$Key}}->{point} ;}
+        if (ref $k eq 'XML::Smart') {
+          $k = defined ${$$tree{$Key}}->{content} ? ${$$tree{$Key}}->{content} : ${$$tree{$Key}}->{point} ;
+        }
         elsif ( ref $k eq 'ARRAY' && $multi_keys{$Key} ) {
           $i = $array_i{$Key}++ if $#{$k} > 0 ;
         }
@@ -249,19 +266,19 @@ sub _data {
         my $k = [$$tree{$Key}] ;
         $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , undef , $$tree{'/nodes'}{$Key} , @stat) ;
       }
-      elsif ("\L$Key\E" eq 'content') {
+      elsif (lc($Key) eq 'content') {
         if ( tied($$tree{$Key}) && $$tree{$Key} =~ /\S/s ) {
           $ident = '' ; $stat[1] += 2 ;
         }
         next if tied($$tree{$Key}) ;
-        $cont .= $$tree{$Key} ;
+        
+        if ( $$tree{$Key} ne '' ) {
+          my $p0 = length($tags) ;
+          $tags .= $$tree{$Key} ;        
+          $cont = [$p0,length($tags)] ;
+        }
       }
       elsif ($Key =~ /^\/\.CONTENT\/\d+$/) { $tags .= $$tree{$Key} ;}
-      elsif ( $Key eq '!--' && (!ref($$tree{$Key}) || ( ref($$tree{$Key}) eq 'HASH' && keys %{$$tree{$Key}} == 1 && (defined $$tree{$Key}{CONTENT} || defined $$tree{$Key}{content}) ) ) ) {
-        my $ct = $$tree{$Key} ;
-        if (ref $$tree{$Key}) { $ct = defined $$tree{$Key}{CONTENT} ? $$tree{$Key}{CONTENT} : $$tree{$Key}{content} ;} ;
-        $cont .= '<!--' . $ct . '-->' ;
-      }
       elsif ( $stat[4] && $$tree{$Key} eq '') { $args_end .= " $Key" ;}
       else {
         my $tp = _data_type($$tree{$Key}) ;
@@ -301,6 +318,9 @@ sub _data {
     }
     
     if ( $cont ne '' ) {
+      my ( $po , $p1 ) = @$cont ;
+      my $cont = substr($tags , $po , $p1) ;
+        
       my $tp = _data_type($cont) ;
       
       if ( $node_type =~ /^(\w+),(\d+),(\d*)$/ ) {
@@ -324,8 +344,17 @@ sub _data {
         $args .= ' dt:dt="binary.base64"' ;
       }
       else { &_add_basic_entity($cont) ;}
+      
+      my $px = $p1 ;
+      while( substr($tags , $px , 1) =~ /\s/ ) { ++$px ;}
+      
+      if ( $px > $p1 ) { substr($tags , $p1 , $px-$p1) = '' ;}
+      
+      substr($tags , $po , $p1) = $cont ;
     }
     
+    ##print "***$tag>> $args,$args_end,$tags,$cont,$stat_1 [@all_keys]\n" ;
+    
     if ($args_end ne '') {
       $args .= $args_end ;
       $args_end = undef ;
@@ -337,14 +366,7 @@ sub _data {
     elsif ($args ne '' && $tags ne '') {
       $$data .= qq`$ident<$tag$args>` if $tag ne '' ;
       $$data .= $tags ;
-      $$data .= $cont ;
-      $$data .= $ident if $cont eq '' ;
-      $$data .= qq`</$tag>` if $tag ne '' ;
-    }
-    elsif ($args ne '' && $cont ne '') {
-      $$data .= qq`$ident<$tag$args>` if $tag ne '' ;
-      $$data .= $cont ;
-      $$data .= $ident if $cont eq '' ;
+      $$data .= $ident if !$cont ;
       $$data .= qq`</$tag>` if $tag ne '' ;
     }
     elsif ($args ne '') {
@@ -353,14 +375,11 @@ sub _data {
     elsif ($tags ne '') {
       $$data .= qq`$ident<$tag>` if $tag ne '' ;
       $$data .= $tags ;
-      $$data .= $cont ;
-      $$data .= $ident if $cont eq '' ;
+      $$data .= $ident if !$cont ;
       $$data .= qq`</$tag>` if $tag ne '' ;
     }
-    elsif ($cont ne '') {
-      $$data .= qq`$ident<$tag>` if $tag ne '' ;
-      $$data .= $cont ;
-      $$data .= qq`</$tag>` if $tag ne '' ;
+    else {
+      $$data .= qq`$ident<$tag></$tag>` if $tag ne '' ;
     }
   }
   elsif (ref($tree) eq 'ARRAY') {
@@ -437,7 +456,7 @@ sub _data {
       my $k = $stat[4] ? $tag : &_check_key($tag) ;
       if    ($stat[3] == 1) { $k = "\L$k\E" ;}
       elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-      delete $$parsed{"$tree"} ;
+      delete $$parsed{"$tree"} if ref($tree) ;
       return " $k=" . &_add_quote($v) ;
     }
     else { $$data .= $tags ;}
@@ -446,18 +465,25 @@ sub _data {
     my $k = $stat[4] ? $tag : &_check_key($tag) ;
     if    ($stat[3] == 1) { $k = "\L$k\E" ;}
     elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-    delete $$parsed{"$tree"} ;
+    delete $$parsed{"$tree"} if ref($tree) ;
     return " $k=" . &_add_quote($$tree) ;
   }
   elsif (ref($tree)) {
     my $k = $stat[4] ? $tag : &_check_key($tag) ;
     if    ($stat[3] == 1) { $k = "\L$k\E" ;}
     elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-    delete $$parsed{"$tree"} ;
+    delete $$parsed{"$tree"} if ref($tree) ;
     return " $k=" . &_add_quote("$tree") ;
   }
+  else {
+    my $k = $stat[4] ? $tag : &_check_key($tag) ;
+    if    ($stat[3] == 1) { $k = "\L$k\E" ;}
+    elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
+    delete $$parsed{"$tree"} if ref($tree) ;
+    return " $k=" . &_add_quote($tree) ;
+  }
 
-  delete $$parsed{"$tree"} ;
+  delete $$parsed{"$tree"} if ref($tree) ;
   return ;
 }
 
